import { r as NarouSearchResult, a as ParamsBaseWithOrder, q as GzipLevel, d as SearchResultFieldNames, S as SearchParams, g as Order, o as NovelTypeParam, m as BuntaiParam, D as DateParam, N as NarouSearchResults, j as BigGenre, G as Genre, F as Fields, u as SearchResultFields, O as OptionalFields, v as SearchResultOptionalFields, y as PickedNarouSearchResult, b as RankingParams, c as RankingHistoryParams, U as UserSearchParams, s as UserSearchResult } from './narou-search-results-DWCJWTC5.cjs';
import { RankingHistoryRawResult } from './ranking-history.cjs';
import { Join } from './util/type.cjs';

type DefaultSearchResultFields = keyof Omit<NarouSearchResult, "weekly_unique" | "noveltype" | "nocgenre" | "xid">;
declare abstract class SearchBuilderBase<TParams extends ParamsBaseWithOrder<TOrder>, TOrder extends string> {
    protected params: TParams;
    protected api: NarouNovel;
    /**
     * constructor
     * @private
     */
    constructor(params: TParams | undefined, api: NarouNovel);
    protected static distinct<T>(array: readonly T[]): T[];
    protected static array2string<T extends string | number>(n: T | readonly T[]): Join<T>;
    /**
     *
     * @return {SearchBuilder} this
     */
    limit(num: number): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    start(num: number): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    page(no: number, count?: number): this;
    /**
     * 出力順序を指定する。指定しない場合は新着順となります。
     * old	古い順
     * @param {TOrder} order 出力順序
     * @return {SearchBuilder} this
     */
    order(order: TOrder): this;
    /**
     * gzip圧縮する。
     *
     * 転送量上限を減らすためにも推奨
     * @param {GzipLevel} level gzip圧縮レベル(1～5)
     * @return {SearchBuilder} this
     */
    gzip(level: GzipLevel): this;
    /**
     * クエリパラメータをセットする
     * @private
     * @return {SearchBuilder} this
     */
    protected set(obj: TParams): this;
    /**
     * クエリパラメータを削除する
     */
    protected unset(key: keyof TParams): this;
}
declare abstract class NovelSearchBuilderBase<T extends SearchResultFieldNames> extends SearchBuilderBase<SearchParams, Order> {
    /**
     * a
     * @return {SearchBuilder} this
     */
    word(word: string): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    notWord(word: string): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    byTitle(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    byOutline(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    byKeyword(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    byAuthor(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isBL(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isGL(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isZankoku(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isTensei(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isTenni(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isTT(): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    length(length: number | readonly number[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    kaiwaritu(num: number): this;
    kaiwaritu(min: number, max: number): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    sasie(num: number | readonly number[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    time(num: number | readonly number[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    ncode(ncodes: string | readonly string[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    type(type: NovelTypeParam): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    buntai(buntai: BuntaiParam | readonly BuntaiParam[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isStop(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isPickup(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    lastUpdate(date: DateParam): this;
    lastUpdate(from: number, to: number): this;
    lastUpdate(from: Date, to: Date): this;
    lastNovelUpdate(date: DateParam): this;
    lastNovelUpdate(from: number, to: number): this;
    lastNovelUpdate(from: Date, to: Date): this;
    /**
     * なろう小説APIへの検索リクエストを実行する
     * @returns {Promise<NarouSearchResults>} 検索結果
     */
    execute(): Promise<NarouSearchResults<NarouSearchResult, T>>;
}
/**
 * 検索ヘルパー
 * @class SearchBuilder
 */
declare class SearchBuilder<T extends keyof NarouSearchResult = DefaultSearchResultFields, TOpt extends keyof NarouSearchResult = never> extends NovelSearchBuilderBase<T | TOpt> {
    /**
     *
     * @return {SearchBuilder} this
     */
    bigGenre(genre: BigGenre | readonly BigGenre[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    notBigGenre(genre: BigGenre | readonly BigGenre[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    genre(genre: Genre | readonly Genre[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    notGenre(genre: Genre | readonly Genre[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    userId(ids: number | readonly number[]): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    isR15(bool?: boolean): this;
    /**
     *
     * @return {SearchBuilder} this
     */
    fields<TFields extends Fields>(fields: TFields | readonly TFields[]): SearchBuilder<SearchResultFields<TFields>, TOpt>;
    opt<TFields extends OptionalFields>(option: TFields | readonly TFields[]): SearchBuilder<T, SearchResultOptionalFields<TFields>>;
}

/**
 * ランキングAPIの結果
 * @see https://dev.syosetu.com/man/rankapi/#output
 */
interface NarouRankingResult {
    /** Nコード */
    ncode: string;
    /** 順位 */
    rank: number;
    /** ポイント */
    pt: number;
}
/**
 * ランキングと小説情報をマージした結果
 */
type RankingResult<T extends SearchResultFieldNames = DefaultSearchResultFields> = Partial<PickedNarouSearchResult<T>> & NarouRankingResult;

/**
 * なろう小説APIへのリクエストパラメータ
 */
type NarouParams = SearchParams | RankingParams | RankingHistoryParams | UserSearchParams;
/**
 * なろう小説APIへのリクエストを実行する
 * @class NarouNovel
 * @private
 */
declare abstract class NarouNovel {
    /**
     * なろうAPIへのAPIリクエストを実行する
     * @param params クエリパラメータ
     * @param endpoint APIエンドポイント
     * @returns 実行結果
     */
    protected abstract execute<T>(params: NarouParams, endpoint: string): Promise<T>;
    /**
     * APIへの検索リクエストを実行する
     * @param params クエリパラメータ
     * @param endpoint APIエンドポイント
     * @returns 検索結果
     */
    protected executeSearch<T extends keyof NarouSearchResult>(params: SearchParams, endpoint?: string): Promise<NarouSearchResults<NarouSearchResult, T>>;
    /**
     * 小説APIへの検索リクエストを実行する
     * @param params クエリパラメータ
     * @returns 検索結果
     * @see https://dev.syosetu.com/man/api/
     */
    executeNovel<T extends keyof NarouSearchResult>(params: SearchParams): Promise<NarouSearchResults<NarouSearchResult, T>>;
    /**
     * R18小説APIへの検索リクエストを実行する
     * @param params クエリパラメータ
     * @returns 検索結果
     * @see https://dev.syosetu.com/xman/api/
     */
    executeNovel18<T extends keyof NarouSearchResult>(params: SearchParams): Promise<NarouSearchResults<NarouSearchResult, T>>;
    /**
     * ランキングAPIへのリクエストを実行する
     * @param params クエリパラメータ
     * @returns ランキング結果
     * @see https://dev.syosetu.com/man/rankapi/
     */
    executeRanking(params: RankingParams): Promise<NarouRankingResult[]>;
    /**
     * 殿堂入りAPiへのリクエストを実行する
     * @param params クエリパラメータ
     * @returns ランキング履歴結果
     * @see https://dev.syosetu.com/man/rankinapi/
     */
    executeRankingHistory(params: RankingHistoryParams): Promise<RankingHistoryRawResult[]>;
    /**
     * ユーザー検索APIへのリクエストを実行する
     * @param params クエリパラメータ
     * @returns 検索結果
     * @see https://dev.syosetu.com/man/userapi/
     */
    executeUserSearch<T extends keyof UserSearchResult>(params: UserSearchParams): Promise<NarouSearchResults<UserSearchResult, T>>;
}

export { type DefaultSearchResultFields as D, NarouNovel as N, type RankingResult as R, SearchBuilder as S, type NarouParams as a, type NarouRankingResult as b, SearchBuilderBase as c, NovelSearchBuilderBase as d };
