import path from "node:path";
import { MagicStringAST } from "magic-string-ast";
import { parseSync } from "oxc-parser";
import process from "node:process";
import { createResolver } from "dts-resolver";
import { getTsconfig, parseTsconfig } from "get-tsconfig";
import { isolatedDeclaration } from "oxc-transform";
import { createRequire } from "node:module";
import Debug from "debug";

//#region src/utils/filename.ts
const RE_JS = /\.([cm]?)jsx?$/;
const RE_TS = /\.([cm]?)tsx?$/;
const RE_DTS = /\.d\.([cm]?)ts$/;
const RE_NODE_MODULES = /[\\/]node_modules[\\/]/;
function filename_js_to_dts(id) {
	return id.replace(RE_JS, ".d.$1ts");
}
function filename_ts_to_dts(id) {
	return id.replace(RE_TS, ".d.$1ts");
}
function filename_dts_to(id, ext) {
	return id.replace(RE_DTS, `.$1${ext}`);
}
function isRelative(id) {
	return path.isAbsolute(id) || id[0] === ".";
}

//#endregion
//#region src/dts-input.ts
function createDtsInputPlugin() {
	return {
		name: "rolldown-plugin-dts:dts-input",
		outputOptions(options) {
			return {
				...options,
				entryFileNames: "[name].ts"
			};
		},
		resolveId: {
			order: "pre",
			handler(id, importer, options) {
				if (options.isEntry) return;
				if (RE_NODE_MODULES.test(id) || !isRelative(id)) return {
					id,
					external: true
				};
			}
		}
	};
}

//#endregion
//#region node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/walker.js
/**
* @typedef { import('estree').Node} Node
* @typedef {{
*   skip: () => void;
*   remove: () => void;
*   replace: (node: Node) => void;
* }} WalkerContext
*/
var WalkerBase = class {
	constructor() {
		/** @type {boolean} */
		this.should_skip = false;
		/** @type {boolean} */
		this.should_remove = false;
		/** @type {Node | null} */
		this.replacement = null;
		/** @type {WalkerContext} */
		this.context = {
			skip: () => this.should_skip = true,
			remove: () => this.should_remove = true,
			replace: (node) => this.replacement = node
		};
	}
	/**
	* @template {Node} Parent
	* @param {Parent | null | undefined} parent
	* @param {keyof Parent | null | undefined} prop
	* @param {number | null | undefined} index
	* @param {Node} node
	*/
	replace(parent, prop, index, node) {
		if (parent && prop) if (index != null)
 /** @type {Array<Node>} */ parent[prop][index] = node;
		else
 /** @type {Node} */ parent[prop] = node;
	}
	/**
	* @template {Node} Parent
	* @param {Parent | null | undefined} parent
	* @param {keyof Parent | null | undefined} prop
	* @param {number | null | undefined} index
	*/
	remove(parent, prop, index) {
		if (parent && prop) if (index !== null && index !== void 0)
 /** @type {Array<Node>} */ parent[prop].splice(index, 1);
		else delete parent[prop];
	}
};

//#endregion
//#region node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/sync.js
/**
* @typedef { import('estree').Node} Node
* @typedef { import('./walker.js').WalkerContext} WalkerContext
* @typedef {(
*    this: WalkerContext,
*    node: Node,
*    parent: Node | null,
*    key: string | number | symbol | null | undefined,
*    index: number | null | undefined
* ) => void} SyncHandler
*/
var SyncWalker = class extends WalkerBase {
	/**
	*
	* @param {SyncHandler} [enter]
	* @param {SyncHandler} [leave]
	*/
	constructor(enter, leave) {
		super();
		/** @type {boolean} */
		this.should_skip = false;
		/** @type {boolean} */
		this.should_remove = false;
		/** @type {Node | null} */
		this.replacement = null;
		/** @type {WalkerContext} */
		this.context = {
			skip: () => this.should_skip = true,
			remove: () => this.should_remove = true,
			replace: (node) => this.replacement = node
		};
		/** @type {SyncHandler | undefined} */
		this.enter = enter;
		/** @type {SyncHandler | undefined} */
		this.leave = leave;
	}
	/**
	* @template {Node} Parent
	* @param {Node} node
	* @param {Parent | null} parent
	* @param {keyof Parent} [prop]
	* @param {number | null} [index]
	* @returns {Node | null}
	*/
	visit(node, parent, prop, index) {
		if (node) {
			if (this.enter) {
				const _should_skip = this.should_skip;
				const _should_remove = this.should_remove;
				const _replacement = this.replacement;
				this.should_skip = false;
				this.should_remove = false;
				this.replacement = null;
				this.enter.call(this.context, node, parent, prop, index);
				if (this.replacement) {
					node = this.replacement;
					this.replace(parent, prop, index, node);
				}
				if (this.should_remove) this.remove(parent, prop, index);
				const skipped = this.should_skip;
				const removed = this.should_remove;
				this.should_skip = _should_skip;
				this.should_remove = _should_remove;
				this.replacement = _replacement;
				if (skipped) return node;
				if (removed) return null;
			}
			/** @type {keyof Node} */
			let key;
			for (key in node) {
				/** @type {unknown} */
				const value = node[key];
				if (value && typeof value === "object") {
					if (Array.isArray(value)) {
						const nodes = value;
						for (let i = 0; i < nodes.length; i += 1) {
							const item = nodes[i];
							if (isNode(item)) {
								if (!this.visit(item, node, key, i)) i--;
							}
						}
					} else if (isNode(value)) this.visit(value, node, key, null);
				}
			}
			if (this.leave) {
				const _replacement = this.replacement;
				const _should_remove = this.should_remove;
				this.replacement = null;
				this.should_remove = false;
				this.leave.call(this.context, node, parent, prop, index);
				if (this.replacement) {
					node = this.replacement;
					this.replace(parent, prop, index, node);
				}
				if (this.should_remove) this.remove(parent, prop, index);
				const removed = this.should_remove;
				this.replacement = _replacement;
				this.should_remove = _should_remove;
				if (removed) return null;
			}
		}
		return node;
	}
};
/**
* Ducktype a node.
*
* @param {unknown} value
* @returns {value is Node}
*/
function isNode(value) {
	return value !== null && typeof value === "object" && "type" in value && typeof value.type === "string";
}

//#endregion
//#region node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/index.js
/**
* @typedef {import('estree').Node} Node
* @typedef {import('./sync.js').SyncHandler} SyncHandler
* @typedef {import('./async.js').AsyncHandler} AsyncHandler
*/
/**
* @param {Node} ast
* @param {{
*   enter?: SyncHandler
*   leave?: SyncHandler
* }} walker
* @returns {Node | null}
*/
function walk(ast, { enter, leave }) {
	const instance = new SyncWalker(enter, leave);
	return instance.visit(ast, null);
}

//#endregion
//#region src/utils/ast.ts
function getIdentifierRange(node, offset = 0) {
	if ("typeAnnotation" in node && node.typeAnnotation) return [node.start + offset, node.typeAnnotation.start + offset];
	return [node.start + offset, node.end + offset];
}

//#endregion
//#region src/utils/magic-string.ts
function overwriteOrAppend(s, range, replacement, suffix) {
	if (range[0] === range[1]) {
		s.appendLeft(range[0], ` ${replacement}`);
		return;
	}
	const original = s.slice(range[0], range[1]);
	if (original !== replacement) s.overwrite(range[0], range[1], replacement + (suffix || ""));
}

//#endregion
//#region src/fake-js.ts
const RE_TYPE = /\btype\b/;
function createFakeJsPlugin({ dtsInput }) {
	let symbolIdx = 0;
	let identifierIdx = 0;
	const symbolMap = new Map();
	const preserveMap = new Map();
	function getIdentifierIndex() {
		return identifierIdx++;
	}
	function register(info) {
		const symbolId = symbolIdx++;
		symbolMap.set(symbolId, info);
		return symbolId;
	}
	function retrieve(symbolId) {
		return symbolMap.get(symbolId);
	}
	return {
		name: "rolldown-plugin-dts:fake-js",
		options: dtsInput ? (options) => {
			return {
				...options,
				resolve: {
					extensions: [
						".d.ts",
						".d.mts",
						".d.cts"
					],
					extensionAlias: {
						".js": [".d.ts"],
						".mjs": [".d.mts"],
						".cjs": [".d.cts"]
					},
					...options.resolve
				}
			};
		} : void 0,
		outputOptions(options) {
			if (options.format === "cjs" || options.format === "commonjs") throw new Error("[rolldown-plugin-dts] Cannot bundle dts files with `cjs` format.");
			return {
				...options,
				entryFileNames: options.entryFileNames ?? (dtsInput ? "[name].ts" : void 0),
				chunkFileNames(chunk) {
					const original = (typeof options.chunkFileNames === "function" ? options.chunkFileNames(chunk) : options.chunkFileNames) || "[name]-[hash].js";
					if (!original.includes(".d") && chunk.name.endsWith(".d")) return filename_js_to_dts(original);
					return original;
				}
			};
		},
		transform: {
			filter: { id: RE_DTS },
			handler(code, id) {
				const { program, comments } = parseSync(id, code);
				const preserved = collectReferenceDirectives(comments);
				preserveMap.set(id, preserved);
				const s = new MagicStringAST(code);
				for (let node of program.body) {
					if (rewriteImportExport(s, node)) continue;
					const sideEffect = node.type === "TSModuleDeclaration" && node.kind !== "namespace";
					const stmt = node;
					const isDefaultExport = node.type === "ExportDefaultDeclaration";
					if ((node.type === "ExportNamedDeclaration" || node.type === "ExportDefaultDeclaration") && node.declaration) node = node.declaration;
					if (node.type === "VariableDeclaration" && node.declarations.length !== 1) throw new Error("Only one declaration is supported");
					if (node.type === "TSDeclareFunction" || node.type.endsWith("Declaration")) {
						const binding = node.type === "VariableDeclaration" ? node.declarations[0].id : node.id;
						const code$1 = s.sliceNode(node);
						const offset = node.start;
						let bindingRange;
						if (sideEffect) bindingRange = [0, 0];
						else if (binding) bindingRange = getIdentifierRange(binding, -offset);
						else if (isDefaultExport) {
							const idx = s.sliceNode(node).indexOf("function") + 8;
							bindingRange = [idx, idx];
						} else continue;
						const depsNodes = collectDependencies(s, node, getIdentifierIndex);
						const depsString = stringifyDependencies(s, depsNodes);
						const depsSymbols = depsNodes.map((dep) => [
							dep.start - offset,
							dep.end - offset,
							dep._suffix
						]);
						const needDeclare = (node.type === "TSEnumDeclaration" || node.type === "ClassDeclaration" || node.type === "FunctionDeclaration" || node.type === "TSDeclareFunction" || node.type === "TSModuleDeclaration" || node.type === "VariableDeclaration") && !node.declare;
						const symbolId = register({
							code: code$1,
							binding: bindingRange,
							deps: depsSymbols,
							needDeclare,
							preserveName: sideEffect
						});
						const runtime = `[${symbolId}, ${depsString}${depsString && sideEffect ? ", " : ""}${sideEffect ? "sideEffect()" : ""}]`;
						const bindingName = sideEffect ? `_${identifierIdx++}` : binding ? s.sliceNode(binding) : "export_default";
						if (isDefaultExport) s.overwriteNode(stmt, `var ${bindingName} = ${runtime};export { ${bindingName} as default }`);
						else s.overwriteNode(node, `var ${bindingName} = ${runtime};`);
					}
				}
				if (!s.hasChanged()) return;
				const str = s.toString();
				return str;
			}
		},
		renderChunk(code, chunk) {
			if (!RE_DTS.test(chunk.fileName)) return;
			const { program } = parseSync(chunk.fileName, code);
			const s = new MagicStringAST(code);
			const comments = new Set();
			for (const id of chunk.moduleIds) {
				const preserveComments = preserveMap.get(id);
				if (preserveComments) {
					preserveComments.forEach((c) => comments.add(c));
					preserveMap.delete(id);
				}
			}
			if (comments.size) s.prepend(`${[...comments].join("\n")}\n`);
			const removedNodes = patchTsNamespace(s, program.body);
			for (const node of program.body) {
				if (removedNodes.has(node)) continue;
				if (patchImportSource(s, node)) continue;
				if (node.type !== "VariableDeclaration" || node.declarations.length !== 1) continue;
				const [decl] = node.declarations;
				if (decl.init?.type !== "ArrayExpression" || !decl.init.elements[0]) {
					s.removeNode(node);
					continue;
				}
				const [symbolIdNode, ...depsNodes] = decl.init.elements;
				if (symbolIdNode?.type !== "Literal" || typeof symbolIdNode.value !== "number") {
					s.removeNode(node);
					continue;
				}
				const symbolId = symbolIdNode.value;
				const { code: code$1, binding, deps, needDeclare, preserveName } = retrieve(symbolId);
				const depsRaw = depsNodes.filter((node$1) => node$1?.type === "ArrowFunctionExpression").map((dep) => s.sliceNode(dep.body));
				const ss = new MagicStringAST(code$1);
				if (!preserveName) overwriteOrAppend(ss, binding, s.sliceNode(decl.id));
				for (const dep of deps) {
					const [start, end, suffix] = dep;
					overwriteOrAppend(ss, [start, end], depsRaw.shift(), suffix);
				}
				if (needDeclare) ss.prepend("declare ");
				s.overwriteNode(node, ss.toString());
			}
			const str = s.toString();
			if (str.trim().length === 0) return "export { };";
			return str;
		}
	};
}
const REFERENCE_RE = /\/\s*<reference\s+(?:path|types)=/;
function collectReferenceDirectives(comment) {
	return comment.filter((c) => REFERENCE_RE.test(c.value)).map((c) => `//${c.value}`);
}
function collectDependencies(s, node, getIdentifierIndex) {
	const deps = new Set();
	walk(node, { leave(node$1) {
		if (node$1.type === "ExportNamedDeclaration") {
			for (const specifier of node$1.specifiers) if (specifier.type === "ExportSpecifier") {
				let _suffix;
				if (specifier.local.start === specifier.exported.start && specifier.local.end === specifier.exported.end) _suffix = ` as ${s.sliceNode(specifier.local)}`;
				addDependency({
					...specifier.local,
					_suffix
				});
			}
		} else if (node$1.type === "TSInterfaceDeclaration" && node$1.extends) for (const heritage of node$1.extends || []) addDependency(heritage.expression);
		else if (node$1.type === "ClassDeclaration") {
			if (node$1.superClass) addDependency(node$1.superClass);
			if (node$1.implements) for (const implement of node$1.implements) addDependency(implement.expression);
		} else if (node$1.type === "MethodDefinition" || node$1.type === "PropertyDefinition" || node$1.type === "TSPropertySignature") {
			if (node$1.computed && isReferenceId(node$1.key)) addDependency(node$1.key);
			if ("value" in node$1 && isReferenceId(node$1.value)) addDependency(node$1.value);
		} else if (node$1.type === "TSTypeReference") addDependency(node$1.typeName);
		else if (node$1.type === "TSTypeQuery") {
			if (node$1.exprName.type !== "TSImportType") addDependency(node$1.exprName);
		} else if (node$1.type === "TSImportType") {
			if (node$1.argument.type !== "TSLiteralType" || node$1.argument.literal.type !== "Literal" || typeof node$1.argument.literal.value !== "string") return;
			const source = node$1.argument.literal.value;
			const imported = node$1.qualifier && resolveTSTypeName(node$1.qualifier);
			const local = importNamespace(s, source, imported && s.sliceNode(imported), getIdentifierIndex);
			addDependency({
				type: "Identifier",
				name: local,
				start: node$1.start,
				end: imported ? imported.end : node$1.end
			});
		}
	} });
	return Array.from(deps);
	function addDependency(node$1) {
		if (node$1.type === "Identifier" && node$1.name === "this") return;
		deps.add(node$1);
	}
}
function resolveTSTypeName(node) {
	if (node.type === "Identifier") return node;
	return resolveTSTypeName(node.left);
}
function isReferenceId(node) {
	return !!node && (node.type === "Identifier" || node.type === "MemberExpression");
}
function stringifyDependencies(s, deps) {
	return deps.map((node) => `() => ${node.type === "Identifier" ? node.name : s.sliceNode(node)}`).join(", ");
}
function patchImportSource(s, node) {
	if ((node.type === "ImportDeclaration" || node.type === "ExportAllDeclaration" || node.type === "ExportNamedDeclaration") && node.source?.value && RE_DTS.test(node.source.value)) {
		s.overwriteNode(node.source, JSON.stringify(filename_dts_to(node.source.value, "js")));
		return true;
	}
}
function patchTsNamespace(s, nodes) {
	const emptyObjectAssignments = new Map();
	const removed = new Set();
	for (const node of nodes) {
		if (node.type === "VariableDeclaration" && node.declarations.length === 1 && node.declarations[0].init?.type === "ObjectExpression" && node.declarations[0].init.properties.length === 0) emptyObjectAssignments.set(s.sliceNode(node.declarations[0].id), node);
		if (node.type !== "ExpressionStatement" || node.expression.type !== "CallExpression" || node.expression.callee.type !== "Identifier" || !node.expression.callee.name.startsWith("__export")) continue;
		const [binding, exports] = node.expression.arguments;
		const bindingText = s.sliceNode(binding);
		if (emptyObjectAssignments.has(bindingText)) {
			const emptyNode = emptyObjectAssignments.get(bindingText);
			s.removeNode(emptyNode);
			emptyObjectAssignments.delete(bindingText);
			removed.add(emptyNode);
		}
		let code = `declare namespace ${bindingText} {
  export { `;
		for (const properties of exports.properties) {
			if (properties.type !== "Property") continue;
			const exported = s.sliceNode(properties.key);
			const local = s.sliceNode(properties.value.body);
			const suffix = exported !== local ? ` as ${exported}` : "";
			code += `${local}${suffix}, `;
		}
		code += `}\n}`;
		s.overwriteNode(node, code);
	}
	return removed;
}
function rewriteImportExport(s, node) {
	if (node.type === "ImportDeclaration" || node.type === "ExportNamedDeclaration" && !node.declaration) {
		for (const specifier of node.specifiers) if (specifier.type === "ImportSpecifier" && specifier.importKind === "type" || specifier.type === "ExportSpecifier" && specifier.exportKind === "type") s.overwriteNode(specifier, s.sliceNode(specifier).replace(RE_TYPE, ""));
		const firstSpecifier = node.specifiers[0];
		const kind = node.type === "ImportDeclaration" ? node.importKind : node.exportKind;
		if (kind === "type" && firstSpecifier) s.overwrite(node.start, firstSpecifier.start, s.slice(node.start, firstSpecifier.start).replace(RE_TYPE, ""));
		return true;
	} else if (node.type === "ExportAllDeclaration") {
		if (node.exportKind === "type") s.overwrite(node.start, node.source.start, s.slice(node.start, node.source.start).replace(RE_TYPE, ""));
		return true;
	} else if (node.type === "TSImportEqualsDeclaration") {
		if (node.moduleReference.type === "TSExternalModuleReference") s.overwriteNode(node, `import ${s.sliceNode(node.id)} from ${s.sliceNode(node.moduleReference.expression)}`);
		return true;
	} else if (node.type === "TSExportAssignment") {
		s.overwriteNode(node, `export { ${s.sliceNode(node.expression)} as default }`);
		return true;
	} else if (node.type === "ExportDefaultDeclaration" && node.declaration.type === "Identifier") {
		s.overwriteNode(node, `export { ${s.sliceNode(node.declaration)} as default }`);
		return true;
	}
}
function importNamespace(s, source, imported, getIdentifierIndex) {
	const local = `_${getIdentifierIndex()}`;
	const specifiers = imported ? `{ ${imported} as ${local} }` : `* as ${local}`;
	s.prepend(`import ${specifiers} from ${JSON.stringify(source)};\n`);
	return local;
}

//#endregion
//#region src/utils/tsc.ts
const debug = Debug("rolldown-plugin-dts:tsc");
let ts;
let formatHost;
function initTs() {
	debug("loading typescript");
	const require = createRequire(import.meta.url);
	ts = require("typescript");
	formatHost = {
		getCurrentDirectory: () => ts.sys.getCurrentDirectory(),
		getNewLine: () => ts.sys.newLine,
		getCanonicalFileName: ts.sys.useCaseSensitiveFileNames ? (f) => f : (f) => f.toLowerCase()
	};
	debug(`loaded typescript: ${ts.version}`);
}
const defaultCompilerOptions = {
	declaration: true,
	noEmit: false,
	emitDeclarationOnly: true,
	noEmitOnError: true,
	checkJs: false,
	declarationMap: false,
	skipLibCheck: true,
	preserveSymlinks: true,
	target: 99,
	resolveJsonModule: true
};
function createOrGetTsModule(programs, compilerOptions, id, isEntry, dtsMap) {
	const program = programs.find((program$1) => {
		if (isEntry) return program$1.getRootFileNames().includes(id);
		return program$1.getSourceFile(id);
	});
	if (program) {
		const sourceFile = program.getSourceFile(id);
		if (sourceFile) return {
			program,
			file: sourceFile
		};
	}
	debug(`create program for module: ${id}`);
	const module = createTsProgram(compilerOptions, dtsMap, id);
	debug(`created program for module: ${id}`);
	programs.push(module.program);
	return module;
}
function createTsProgram(compilerOptions, dtsMap, id) {
	const overrideCompilerOptions = ts.convertCompilerOptionsFromJson(compilerOptions, ".").options;
	const options = {
		...defaultCompilerOptions,
		...overrideCompilerOptions
	};
	const host = ts.createCompilerHost(options, true);
	const { readFile: _readFile, fileExists: _fileExists } = host;
	host.fileExists = (fileName) => {
		const module = getTsModule(dtsMap, fileName);
		if (module) return true;
		if (debug.enabled && !RE_NODE_MODULES.test(fileName)) debug(`file exists from fs: ${fileName}`);
		return _fileExists(fileName);
	};
	host.readFile = (fileName) => {
		const module = getTsModule(dtsMap, fileName);
		if (module) return module.code;
		if (debug.enabled && !RE_NODE_MODULES.test(fileName)) debug(`read file from fs: ${fileName}`);
		return _readFile(fileName);
	};
	const entries = Array.from(dtsMap.values()).filter((v) => v.isEntry).map((v) => v.id);
	const program = ts.createProgram(Array.from(new Set([id, ...entries])), options, host);
	const sourceFile = program.getSourceFile(id);
	if (!sourceFile) throw new Error(`Source file not found: ${id}`);
	return {
		program,
		file: sourceFile
	};
}
function tscEmit(module) {
	const { program, file } = module;
	let dtsCode;
	const { emitSkipped, diagnostics } = program.emit(
		file,
		(_, code) => {
			debug(`emit dts: ${file.fileName}`);
			dtsCode = code;
		},
		void 0,
		true,
		void 0,
		// @ts-expect-error private API: forceDtsEmit
		true
);
	if (emitSkipped && diagnostics.length) return { error: ts.formatDiagnostics(diagnostics, formatHost) };
	return { code: dtsCode };
}
function getTsModule(dtsMap, tsId) {
	const module = Array.from(dtsMap.values()).find((dts$1) => dts$1.id === tsId);
	if (!module) return;
	return module;
}

//#endregion
//#region src/generate.ts
const meta = { dtsFile: true };
function createGeneratePlugin({ tsconfig, compilerOptions, isolatedDeclarations, resolve = false, emitDtsOnly = false }) {
	const dtsMap = new Map();
	function resolveOptions(cwd) {
		if (tsconfig === true || tsconfig == null) {
			const { config, path: path$1 } = getTsconfig(cwd) || {};
			tsconfig = path$1;
			compilerOptions = {
				...config?.compilerOptions,
				...compilerOptions
			};
		} else if (typeof tsconfig === "string") {
			tsconfig = path.resolve(cwd || process.cwd(), tsconfig);
			const config = parseTsconfig(tsconfig);
			compilerOptions = {
				...config.compilerOptions,
				...compilerOptions
			};
		}
		if (isolatedDeclarations == null) isolatedDeclarations = !!compilerOptions?.isolatedDeclarations;
		if (isolatedDeclarations === true) isolatedDeclarations = {};
		if (isolatedDeclarations && isolatedDeclarations.stripInternal == null) isolatedDeclarations.stripInternal = !!compilerOptions?.stripInternal;
	}
	/**
	* A map of input id to output file name
	*
	* @example
	*
	* inputAlias = new Map([
	*   ['/absolute/path/to/src/source_file.ts', 'dist/foo/index'],
	* ])
	*/
	const inputAliasMap = new Map();
	let resolver;
	let programs = [];
	return {
		name: "rolldown-plugin-dts:generate",
		async buildStart(options) {
			resolveOptions(options.cwd);
			resolver = createResolver({ tsconfig: tsconfig ? tsconfig : void 0 });
			if (!isolatedDeclarations) initTs();
			if (!Array.isArray(options.input)) for (const [name, id] of Object.entries(options.input)) {
				let resolved = await this.resolve(id, void 0, { skipSelf: true });
				resolved ||= await this.resolve(`./${id}`, void 0, { skipSelf: true });
				inputAliasMap.set(resolved?.id || id, name);
			}
		},
		outputOptions(options) {
			return {
				...options,
				entryFileNames(chunk) {
					const original = (typeof options.entryFileNames === "function" ? options.entryFileNames(chunk) : options.entryFileNames) || "[name].js";
					if (!original.includes(".d") && chunk.name.endsWith(".d")) return original.replace(RE_JS, ".$1ts");
					return original;
				}
			};
		},
		transform: {
			order: "pre",
			filter: { id: {
				include: [RE_TS],
				exclude: [RE_DTS, RE_NODE_MODULES]
			} },
			handler(code, id) {
				const mod = this.getModuleInfo(id);
				const isEntry = !!mod?.isEntry;
				const dtsId = filename_ts_to_dts(id);
				dtsMap.set(dtsId, {
					code,
					id,
					isEntry
				});
				if (isEntry) {
					const name = inputAliasMap.get(id);
					this.emitFile({
						type: "chunk",
						id: dtsId,
						name: name ? `${name}.d` : void 0
					});
				}
				if (emitDtsOnly) return "export { }";
			}
		},
		resolveId: {
			order: "pre",
			async handler(id, importer, options) {
				if (dtsMap.has(id)) return {
					id,
					meta
				};
				if (!importer || !this.getModuleInfo(importer)?.meta.dtsFile) return;
				if (RE_DTS.test(id)) {
					const resolution$1 = await this.resolve(id, importer, options);
					if (!resolution$1) return;
					return {
						...resolution$1,
						meta
					};
				}
				if (RE_NODE_MODULES.test(importer)) {
					const resolution$1 = resolver(id, importer);
					if (resolution$1) return {
						id: resolution$1,
						meta
					};
				}
				let resolution = await this.resolve(id, filename_dts_to(importer, "ts"));
				if (RE_NODE_MODULES.test(resolution?.id || id) || !isRelative(resolution?.id || id)) {
					let shouldResolve;
					if (typeof resolve === "boolean") shouldResolve = resolve;
					else shouldResolve = resolve.some((pattern) => typeof pattern === "string" ? id === pattern : pattern.test(id));
					if (shouldResolve) {
						const resolution$1 = resolver(id, importer);
						if (resolution$1) return {
							id: resolution$1,
							meta
						};
					} else return {
						id,
						external: true,
						meta
					};
				}
				if (!resolution || resolution.external) return resolution;
				let dtsId;
				if (RE_JS.test(resolution.id)) {
					resolution = await this.resolve(filename_js_to_dts(resolution.id), importer, { skipSelf: false });
					if (!resolution) return;
					dtsId = resolution.id;
				} else {
					dtsId = filename_ts_to_dts(resolution.id);
					if (dtsMap.has(dtsId)) return {
						id: dtsId,
						meta
					};
				}
				await this.load(resolution);
				if (RE_DTS.test(resolution.id) || dtsMap.has(dtsId)) return {
					id: dtsId,
					meta
				};
			}
		},
		load: {
			filter: { id: {
				include: [RE_DTS],
				exclude: [RE_NODE_MODULES]
			} },
			handler(dtsId) {
				if (!dtsMap.has(dtsId)) return;
				const { code, id, isEntry } = dtsMap.get(dtsId);
				let dtsCode;
				if (isolatedDeclarations) {
					const result = isolatedDeclaration(id, code, isolatedDeclarations === true ? {} : isolatedDeclarations);
					if (result.errors.length) {
						const [error] = result.errors;
						return this.error({
							message: error.message,
							frame: error.codeframe
						});
					}
					dtsCode = result.code;
				} else {
					const module = createOrGetTsModule(programs, compilerOptions, id, isEntry, dtsMap);
					const result = tscEmit(module);
					if (result.error) return this.error(result.error);
					dtsCode = result.code;
				}
				if (!dtsCode) return this.error(new Error(`Failed to generate dts for ${id}`));
				return {
					code: dtsCode,
					moduleSideEffects: false
				};
			}
		},
		generateBundle: emitDtsOnly ? (options, bundle) => {
			for (const fileName of Object.keys(bundle)) if (!RE_DTS.test(fileName)) delete bundle[fileName];
		} : void 0,
		buildEnd() {
			programs = [];
		}
	};
}

//#endregion
//#region src/index.ts
function dts(options = {}) {
	const plugins = [];
	if (options.dtsInput) plugins.push(createDtsInputPlugin());
	else plugins.push(createGeneratePlugin(options));
	plugins.push(createFakeJsPlugin(options));
	return plugins;
}

//#endregion
export { createFakeJsPlugin, createGeneratePlugin, dts };